/*
Copyright 2019 Rancher Labs.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by main. DO NOT EDIT.

package v1

import (
	"context"

	v1 "github.com/rancher/rio/pkg/apis/admin.rio.cattle.io/v1"
	clientset "github.com/rancher/rio/pkg/generated/clientset/versioned/typed/admin.rio.cattle.io/v1"
	informers "github.com/rancher/rio/pkg/generated/informers/externalversions/admin.rio.cattle.io/v1"
	listers "github.com/rancher/rio/pkg/generated/listers/admin.rio.cattle.io/v1"
	"github.com/rancher/wrangler/pkg/generic"
	"k8s.io/apimachinery/pkg/api/equality"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/labels"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/apimachinery/pkg/types"
	utilruntime "k8s.io/apimachinery/pkg/util/runtime"
	"k8s.io/apimachinery/pkg/watch"
	"k8s.io/client-go/tools/cache"
)

type FeatureHandler func(string, *v1.Feature) (*v1.Feature, error)

type FeatureController interface {
	FeatureClient

	OnChange(ctx context.Context, name string, sync FeatureHandler)
	OnRemove(ctx context.Context, name string, sync FeatureHandler)
	Enqueue(namespace, name string)

	Cache() FeatureCache

	Informer() cache.SharedIndexInformer
	GroupVersionKind() schema.GroupVersionKind

	AddGenericHandler(ctx context.Context, name string, handler generic.Handler)
	AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler)
	Updater() generic.Updater
}

type FeatureClient interface {
	Create(*v1.Feature) (*v1.Feature, error)
	Update(*v1.Feature) (*v1.Feature, error)
	UpdateStatus(*v1.Feature) (*v1.Feature, error)
	Delete(namespace, name string, options *metav1.DeleteOptions) error
	Get(namespace, name string, options metav1.GetOptions) (*v1.Feature, error)
	List(namespace string, opts metav1.ListOptions) (*v1.FeatureList, error)
	Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error)
	Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.Feature, err error)
}

type FeatureCache interface {
	Get(namespace, name string) (*v1.Feature, error)
	List(namespace string, selector labels.Selector) ([]*v1.Feature, error)

	AddIndexer(indexName string, indexer FeatureIndexer)
	GetByIndex(indexName, key string) ([]*v1.Feature, error)
}

type FeatureIndexer func(obj *v1.Feature) ([]string, error)

type featureController struct {
	controllerManager *generic.ControllerManager
	clientGetter      clientset.FeaturesGetter
	informer          informers.FeatureInformer
	gvk               schema.GroupVersionKind
}

func NewFeatureController(gvk schema.GroupVersionKind, controllerManager *generic.ControllerManager, clientGetter clientset.FeaturesGetter, informer informers.FeatureInformer) FeatureController {
	return &featureController{
		controllerManager: controllerManager,
		clientGetter:      clientGetter,
		informer:          informer,
		gvk:               gvk,
	}
}

func FromFeatureHandlerToHandler(sync FeatureHandler) generic.Handler {
	return func(key string, obj runtime.Object) (ret runtime.Object, err error) {
		var v *v1.Feature
		if obj == nil {
			v, err = sync(key, nil)
		} else {
			v, err = sync(key, obj.(*v1.Feature))
		}
		if v == nil {
			return nil, err
		}
		return v, err
	}
}

func (c *featureController) Updater() generic.Updater {
	return func(obj runtime.Object) (runtime.Object, error) {
		newObj, err := c.Update(obj.(*v1.Feature))
		if newObj == nil {
			return nil, err
		}
		return newObj, err
	}
}

func UpdateFeatureOnChange(updater generic.Updater, handler FeatureHandler) FeatureHandler {
	return func(key string, obj *v1.Feature) (*v1.Feature, error) {
		if obj == nil {
			return handler(key, nil)
		}

		copyObj := obj.DeepCopy()
		newObj, err := handler(key, copyObj)
		if newObj != nil {
			copyObj = newObj
		}
		if obj.ResourceVersion == copyObj.ResourceVersion && !equality.Semantic.DeepEqual(obj, copyObj) {
			newObj, err := updater(copyObj)
			if newObj != nil && err == nil {
				copyObj = newObj.(*v1.Feature)
			}
		}

		return copyObj, err
	}
}

func (c *featureController) AddGenericHandler(ctx context.Context, name string, handler generic.Handler) {
	c.controllerManager.AddHandler(ctx, c.gvk, c.informer.Informer(), name, handler)
}

func (c *featureController) AddGenericRemoveHandler(ctx context.Context, name string, handler generic.Handler) {
	removeHandler := generic.NewRemoveHandler(name, c.Updater(), handler)
	c.controllerManager.AddHandler(ctx, c.gvk, c.informer.Informer(), name, removeHandler)
}

func (c *featureController) OnChange(ctx context.Context, name string, sync FeatureHandler) {
	c.AddGenericHandler(ctx, name, FromFeatureHandlerToHandler(sync))
}

func (c *featureController) OnRemove(ctx context.Context, name string, sync FeatureHandler) {
	removeHandler := generic.NewRemoveHandler(name, c.Updater(), FromFeatureHandlerToHandler(sync))
	c.AddGenericHandler(ctx, name, removeHandler)
}

func (c *featureController) Enqueue(namespace, name string) {
	c.controllerManager.Enqueue(c.gvk, c.informer.Informer(), namespace, name)
}

func (c *featureController) Informer() cache.SharedIndexInformer {
	return c.informer.Informer()
}

func (c *featureController) GroupVersionKind() schema.GroupVersionKind {
	return c.gvk
}

func (c *featureController) Cache() FeatureCache {
	return &featureCache{
		lister:  c.informer.Lister(),
		indexer: c.informer.Informer().GetIndexer(),
	}
}

func (c *featureController) Create(obj *v1.Feature) (*v1.Feature, error) {
	return c.clientGetter.Features(obj.Namespace).Create(obj)
}

func (c *featureController) Update(obj *v1.Feature) (*v1.Feature, error) {
	return c.clientGetter.Features(obj.Namespace).Update(obj)
}

func (c *featureController) UpdateStatus(obj *v1.Feature) (*v1.Feature, error) {
	return c.clientGetter.Features(obj.Namespace).UpdateStatus(obj)
}

func (c *featureController) Delete(namespace, name string, options *metav1.DeleteOptions) error {
	return c.clientGetter.Features(namespace).Delete(name, options)
}

func (c *featureController) Get(namespace, name string, options metav1.GetOptions) (*v1.Feature, error) {
	return c.clientGetter.Features(namespace).Get(name, options)
}

func (c *featureController) List(namespace string, opts metav1.ListOptions) (*v1.FeatureList, error) {
	return c.clientGetter.Features(namespace).List(opts)
}

func (c *featureController) Watch(namespace string, opts metav1.ListOptions) (watch.Interface, error) {
	return c.clientGetter.Features(namespace).Watch(opts)
}

func (c *featureController) Patch(namespace, name string, pt types.PatchType, data []byte, subresources ...string) (result *v1.Feature, err error) {
	return c.clientGetter.Features(namespace).Patch(name, pt, data, subresources...)
}

type featureCache struct {
	lister  listers.FeatureLister
	indexer cache.Indexer
}

func (c *featureCache) Get(namespace, name string) (*v1.Feature, error) {
	return c.lister.Features(namespace).Get(name)
}

func (c *featureCache) List(namespace string, selector labels.Selector) ([]*v1.Feature, error) {
	return c.lister.Features(namespace).List(selector)
}

func (c *featureCache) AddIndexer(indexName string, indexer FeatureIndexer) {
	utilruntime.Must(c.indexer.AddIndexers(map[string]cache.IndexFunc{
		indexName: func(obj interface{}) (strings []string, e error) {
			return indexer(obj.(*v1.Feature))
		},
	}))
}

func (c *featureCache) GetByIndex(indexName, key string) (result []*v1.Feature, err error) {
	objs, err := c.indexer.ByIndex(indexName, key)
	if err != nil {
		return nil, err
	}
	for _, obj := range objs {
		result = append(result, obj.(*v1.Feature))
	}
	return result, nil
}
